function schedule_draw(state, context) {
    if (!state.timers.raf) {
        window.requestAnimationFrame(() => draw(state, context));
        state.timers.raf = true;
    }
}

function draw(state, context) {
    state.timers.raf = false;

    const gl = context.gl;
    const width = window.innerWidth;
    const height = window.innerHeight;
    
    let locations;
    let buffers;

    gl.viewport(0, 0, context.canvas.width, context.canvas.height);
    gl.clearColor(context.bgcolor.r, context.bgcolor.g, context.bgcolor.b, 1);
    gl.clear(gl.COLOR_BUFFER_BIT);

    // Images
    locations = context.locations['quad'];
    buffers = context.buffers['quad'];

    gl.useProgram(context.programs['quad']);
    
    gl.enableVertexAttribArray(locations['a_pos']);
    gl.enableVertexAttribArray(locations['a_texcoord']);

    gl.uniform2f(locations['u_res'], context.canvas.width, context.canvas.height);
    gl.uniform2f(locations['u_scale'], state.canvas.zoom, state.canvas.zoom);
    gl.uniform2f(locations['u_translation'], state.canvas.offset.x, state.canvas.offset.y);
    gl.uniform1i(locations['u_texture'], 0);

    gl.bindBuffer(gl.ARRAY_BUFFER, buffers['b_pos']);
    gl.vertexAttribPointer(locations['a_pos'], 2, gl.FLOAT, false, 0, 0);
    gl.bufferData(gl.ARRAY_BUFFER, context.quad_positions_f32, gl.STATIC_DRAW);

    gl.bindBuffer(gl.ARRAY_BUFFER, buffers['b_texcoord']);
    gl.vertexAttribPointer(locations['a_texcoord'], 2, gl.FLOAT, false, 0, 0);
    gl.bufferData(gl.ARRAY_BUFFER, context.quad_texcoords_f32, gl.STATIC_DRAW);

    const count = Object.keys(context.textures).length;
    let active_image_index = -1;

    gl.uniform1i(locations['u_outline'], 0);

    for (let key = 0; key < count; ++key) {
        if (context.textures[key].image_id === context.active_image) {
            active_image_index = key;
            continue;
        }

        gl.bindTexture(gl.TEXTURE_2D, context.textures[key].texture);
        gl.drawArrays(gl.TRIANGLES, key * 6, 6);
    }

    if (active_image_index !== -1) {
        gl.uniform1i(locations['u_outline'], 1);
        gl.bindTexture(gl.TEXTURE_2D, context.textures[active_image_index].texture);
        gl.drawArrays(gl.TRIANGLES, active_image_index * 6, 6);
    }

    // Strokes
    locations = context.locations['stroke'];
    buffers = context.buffers['stroke'];
    
    gl.useProgram(context.programs['stroke']);
    
    gl.enableVertexAttribArray(locations['a_type']);
    gl.enableVertexAttribArray(locations['a_pos']);
    gl.enableVertexAttribArray(locations['a_texcoord']);
    gl.enableVertexAttribArray(locations['a_color']);

    gl.uniform2f(locations['u_res'], context.canvas.width, context.canvas.height);
    gl.uniform2f(locations['u_scale'], state.canvas.zoom, state.canvas.zoom);
    gl.uniform2f(locations['u_translation'], state.canvas.offset.x, state.canvas.offset.y);

    gl.bindBuffer(gl.ARRAY_BUFFER, buffers['b_packed']);
    
    gl.vertexAttribPointer(locations['a_pos'],      2, gl.FLOAT,         false, config.bytes_per_point, 0);
    gl.vertexAttribPointer(locations['a_texcoord'], 2, gl.FLOAT,         false, config.bytes_per_point, 8);
    gl.vertexAttribPointer(locations['a_color'],    3, gl.UNSIGNED_BYTE, true,  config.bytes_per_point, 16);
    gl.vertexAttribPointer(locations['a_type'],     1, gl.UNSIGNED_BYTE, false, config.bytes_per_point, 19);

    gl.bufferData(gl.ARRAY_BUFFER, context.static_stroke_serializer.buffer, gl.STATIC_DRAW);
    gl.drawArrays(gl.TRIANGLES, 0, context.static_stroke_serializer.offset / config.bytes_per_point);

    gl.bufferData(gl.ARRAY_BUFFER, context.dynamic_stroke_serializer.buffer, gl.STATIC_DRAW);
    gl.drawArrays(gl.TRIANGLES, 0, context.dynamic_stroke_serializer.offset / config.bytes_per_point);
}